### Cropping device ############################################################

##' @title Cropping Device
##' @param file output file name (including extension) or NULL (no cropping)
##' @param warn logical indicating whether a warning should be given in case
##'        of missing software
##' @param which see dev.off()
##' @param post.cmd character string appended to the main underlying commands
##'        (epstool, pdfcrop, mogrify)
##' @param ... additional arguments passed to system()
##' @return invisible()
##' @author Marius Hofert
##' @note pdfcrop ignores ignore.stdout = TRUE, ignore.stderr = TRUE, hence
##'       the solution with 1>/dev/null 2>&1
dev.off.crop <- function(file = NULL, warn = TRUE, which = dev.cur(),
                         post.cmd = "1>/dev/null 2>&1", ...)
{
    ## Close device
    dev.off(which = which)

    ## Check
    stopifnot(length(file) == 1) # otherwise 'switch(file.ext.low, ...)' below fails
    if(!is.null(file)) { # cropping should be done
        if(.Platform$OS.type != "unix") {
            if(warn)
                warning("dev.off.crop() only works on Unix(-like) systems; no cropping is done.")
        } else { # Unix-like systems

            ## Cropping
            ## fails for relative paths: f <- file.path(getwd(), file) # get whole path (incl. file name) to file
            file.ext <- file_ext(file) # could be pdf, Pdf, PDF...
            file.ext.low <- tolower(file.ext) # file ending in lower case
            switch(file.ext.low,
                   "ps" = {
                       stop("For cropping, choose .eps instead of .ps as output file ending.")
                       ## Otherwise epstool fails (as input is .ps instead of .eps) and we would
                       ## thus need to call ps2eps (which already does the cropping), so the output
                       ## file is .eps either way (renaming it to .ps loses the cropping!).
                   },
                   "eps" = {
                       if(!command.exists("epstool")) { # check if epstool is available
                           if(warn)
                               warning("Command 'epstool' for cropping PS files not found; no cropping is done.")
                       } else { # crop
                           ## Note: Although one can call epstool with the same output file name as
                           ##       the input file, it produces an empty plot, hence the approach
                           ##       with renaming.
                           file. <- paste0(file,".eps") # output file name that ps2eps will produce
                           cmd <- paste0("ps2eps --force \"",file,"\" ",post.cmd,"; mv \"",file.,"\" \"",file,"\"")
                           ## OLD (failed with "Input file ... is not EPSF."):
                           ## cmd <- paste0("epstool --copy --bbox \"",file,"\" \"",file.,"\" ",post.cmd,"; mv \"",file.,"\" \"",file,"\"")
                           system(cmd, ...)
                       }
                   },
                   "pdf" = {
                       if(!command.exists("pdfcrop")) { # check if pdfcrop is available
                           if(warn)
                               warning("Command 'pdfcrop' for cropping PDF files not found; no cropping is done.")
                       } else { # crop
                           cmd <- paste0("pdfcrop --pdftexcmd pdftex \"",file,"\" \"",file,"\" ",post.cmd)
                           system(cmd, ...)
                       }
                   },
                   "png" = {
                       if(!command.exists("mogrify")) { # check if mogrify is available
                           if(warn)
                               warning("Command 'mogrify' for cropping PNG files not found; no cropping is done.")
                       } else { # crop
                           cmd <- paste0("mogrify -trim \"",file,"\" \"",file,"\" ",post.cmd) # expected output "profile 'icc': 'RGB ': RGB color space not permitted on grayscale PNG", see https://stackoverflow.com/questions/48628878/error-in-converting-images-in-imagemagick
                           system(cmd, ...)
                       }
                   },
                   stop("Non-supported file extension."))

        }
    }

    ## Return
    invisible()
}
